using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Blazored.LocalStorage;
using System.Text;
using System.Text.Json;

namespace Datepicker
{
    public class JwtDecode
    {
        // TODO: Figure out how to get the JWT Auth .net libs into blazor withoout totaly screwing things up
        // and replace all the JwtClaim names in this file with proper constants.

        private string _header { get; set; }
        private string _payload { get; set; }
        private string _token { get; set; }
        private readonly ILocalStorageService _localStorage;
        private string _authTokenName { get; set; }

        public JwtDecode(ILocalStorageService localStorage)
        {
            _localStorage = localStorage;
        }

        private void DecodeToken()
        {
            string[] parts = _token.Split('.');
            _header = parts[0];
            _payload = parts[1];

            if (_header.Length % 4 != 0) // B64 strings must be a length that is a multiple of 4 to decode them
            {
                var lengthToBe = _header.Length.GetNextHighestMultiple(4);
                _header = _header.PadRight(lengthToBe, '=');
            }

            if (_payload.Length % 4 != 0) // B64 strings must be a length that is a multiple of 4 to decode them
            {
                var lengthToBe = _payload.Length.GetNextHighestMultiple(4);
                _payload = _payload.PadRight(lengthToBe, '=');
            }

        }

        public async Task<bool> LoadToken(string tokenName)
        {
            _authTokenName = tokenName;
            _token = await _localStorage.GetItemAsync<string>(_authTokenName);
            return true;
        }

        public Dictionary<string, object> GetPayload()
        {
            DecodeToken();

            var payload = Encoding.UTF8.GetString(Convert.FromBase64String(_payload));
            var data = JsonSerializer.Deserialize<Dictionary<string, object>>(payload);

            // Testing
            //foreach (var key in data)
            //{
            //    Console.WriteLine($"PAYLOAD: {key.Key} = {key.Value} (TYP: {key.Value.GetType().ToString()})");
            //}

            return data;
        }

        public Dictionary<string, object> GetHeader()
        {
            DecodeToken();

            var header = Encoding.UTF8.GetString(Convert.FromBase64String(_header));
            var data = JsonSerializer.Deserialize<Dictionary<string, object>>(header);

            // Testing
            //foreach (var key in data)
            //{
            //  Console.WriteLine($"HEADER: {key.Key} = {key.Value}");
            //}

            return data;
        }

        public string GetUserName()
        {
            string result = "Not Set";
            string nameClaim = "UserName";

            Dictionary<string, object> payload = GetPayload();
            if (payload.ContainsKey(nameClaim))
            {
                result = Convert.ToString(payload[nameClaim]);
            }
            return result;
        }

        public Int32 GetUserID()
        {
            Int32 result = 0;
            string nameClaim = "UserID";

            Dictionary<string, object> payload = GetPayload();
            if (payload.ContainsKey(nameClaim))
            {
                result = Convert.ToInt32(Convert.ToString(payload[nameClaim]));
            }

            return result;
        }


        public bool HasTokenExpired()
        {
            bool result = false; // Always default to NO
            string expiryClaim = "exp";

            Dictionary<string, object> payload = GetPayload();
            if (payload.ContainsKey(expiryClaim))
            {
                double timeStamp = Convert.ToDouble(Convert.ToString(payload[expiryClaim]));
                DateTime expiryTime = new DateTime(1970, 1, 1, 0, 0, 0, 0, DateTimeKind.Utc);
                expiryTime = expiryTime.AddSeconds(timeStamp);
                if (expiryTime < DateTime.UtcNow)
                {
                    result = true;
                }
            }

            return result;
        }

        public string GetIssuer()
        {
            string result = "NOT SET";
            string issuerClaim = "iss";

            Dictionary<string, object> payload = GetPayload();
            if (payload.ContainsKey(issuerClaim))
            {
                result = payload[issuerClaim] as string;
            }

            return result;
        }
    }
}